﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Bunny : MonoBehaviour
{
    // NO STATICS PLEASE otherwise reloading the scene will fuck up the game
    public static int count { get { return Globals.instance.bunnyCount; } set { Globals.instance.bunnyCount = value; } }
    public static int aliveCount { get { return Globals.instance.aliveBunnyCount; } set { Globals.instance.aliveBunnyCount = value; } }
    public static int bornCount { get { return Globals.instance.bornBunnyCount; } set { Globals.instance.bornBunnyCount = value; } }

    private const float HOP_DURATION_BASE = 0.2f;
    private const float HOP_HEIGHT_BASE = 0.4f;
    private const float GROWTH_RATE = 0.1f;
    private const float PANIC_RADIUS = 1.0f;

    public Sprite hungrySprite;
    public Sprite ateSprite;
    public Sprite deadSprite;
    public AudioClip birthSound;
    public AudioClip eatSound;
    public ParticleSystem speedParticles;
    public bool randomizeColor = false;

    private Vector2 _hopStartPos;
    private Vector2 _hopTargetPos;
    private float _hopTime;
    private float _hopDuration = HOP_DURATION_BASE;
    private bool _isHopping;
    private float _hopHeight = HOP_HEIGHT_BASE;
    private bool _isRotativeHop = false;

    private List<CarrotType> _carrots = new List<CarrotType>();

    //private bool _ate = false;
    private bool _alive = true;
    public bool isAdult = true;

    public bool ate
    {
        get { return _carrots.Count > 0; }
        //set
        //{
        //    if(value != _ate)
        //    {
        //        _ate = value;
        //        GetComponent<SpriteRenderer>().sprite = _ate ? ateSprite : hungrySprite;
        //    }
        //}
    }

    public bool alive
    {
        get { return _alive; }
    }

    public bool isHopping
    {
        get { return _isHopping; }
    }

	void Awake ()
    {
        ++count;
        ++aliveCount;
        ++bornCount;
	}

    void Start()
    {
        if(randomizeColor)
        {
            RandomizeColor();
        }

        if (speedParticles == null)
            speedParticles = GetComponent<ParticleSystem>();

        RandomizeEars();
    }

    void RandomizeEars()
    {
        var s = GetComponent<SpriteRenderer>();
        if(Random.Range(0f, 1f) < 0.5f)
        {
            s.sprite = ateSprite;
        }
        else
        {
            s.sprite = hungrySprite;
        }
    }
	
	void Update ()
    {
        Vector2 pos = transform.position;
        Vector3 scale = transform.localScale;

        // Growth
        if(_alive && !isAdult)
        {
            scale.x += GROWTH_RATE * Time.deltaTime;
            if(scale.x >= 1f)
            {
                scale.x = 1f;
                isAdult = true;
            }
            scale.y = scale.x;
            transform.localScale = scale;
        }

        // Hopping
        if(_isHopping)
        {
            _hopTime += Time.deltaTime;
            if (_hopTime > _hopDuration)
            {
                _hopTime = _hopDuration;
                _isHopping = false;

                if(_alive && _isRotativeHop)
                {
                    transform.localRotation = Quaternion.identity;
                }
            }

            float hopRatio = _hopTime / _hopDuration;
            //Debug.DrawLine(_hopStartPos, _hopTargetPos);

            pos = Vector2.Lerp(_hopStartPos, _hopTargetPos, hopRatio) + new Vector2(0f, _hopHeight) * HopCurve(hopRatio);

            transform.position = pos;

            if (_isHopping && _isRotativeHop)
            {
                transform.Rotate(Vector3.forward, Time.deltaTime * 1200f);
            }
        }
	}

    void OnTriggerEnter2D(Collider2D other)
    {
        if(_carrots.Count > 0)
        {
            // LOVE
            Bunny otherBunny = other.GetComponent<Bunny>();
            if (otherBunny != null && canMakeLove && otherBunny.canMakeLove)
            {
                MakeLove(otherBunny);
            }
        }
        // EAT
        if (canEat && other.GetComponent<Carrot>() != null)
        {
            Eat(other.gameObject);
        }
    }

    bool canEat
    {
        get { return _alive && GetComponent<BunnyController>() != null;/* && !_ate*/; }
    }

    bool canMakeLove
    {
        get { return _alive && isAdult /*&& _ate*/; }
    }

    void MakeLove(Bunny otherBunny)
    {
        ConsumeCarrot();
        otherBunny.ConsumeCarrot();
        SpawnMiniMe(otherBunny);
    }

    void ConsumeCarrot()
    {
        if(_carrots.Count > 0)
        {
            _carrots.RemoveAt(0);
        }
    }

    void AddCarrot(CarrotType t)
    {
        _carrots.Add(t);
    }

    void RandomizeColor()
    {
        // Multiple colors!
        float rc = Mathf2.Sq(Random.Range(0f, 1f));
        Color c = Color.Lerp(
            Color.white,
            new Color(137f / 255f, 89f / 255f, 42f / 255f),
            rc
        );
        c.a = 1f;
        GetComponent<SpriteRenderer>().color = c;
    }

    void SpawnMiniMe(Bunny otherBunny=null)
    {
        GameManager.instance.GetComponent<AudioSource>().PlayOneShot(birthSound);
        Vector3 pos = transform.position;
        GameObject obj = (GameObject)Instantiate(Globals.instance.bunnyPrefab, pos, Quaternion.identity);
        float s = 0.5f;
        obj.transform.localScale = new Vector3(s, s);
        Bunny b = obj.GetComponent<Bunny>();
        b.isAdult = false;
        b.RandomHop();
        
        if(Random.Range(0f, 1f) < 0.1f)
        {
            b.RandomizeColor();
        }
        else
        {
            if(otherBunny != null)
                b.GetComponent<SpriteRenderer>().color = otherBunny.GetComponent<SpriteRenderer>().color;
        }

        GameManager.instance.OnBunnyBorn();
    }

    void Eat(GameObject obj)
    {
        Carrot c = obj.GetComponent<Carrot>();
        if(c != null)
        {
            c.OnPickup(this);
            AddCarrot(c.type);
        }
        Destroy(obj);
        GameManager.instance.GetComponent<AudioSource>().PlayOneShot(eatSound, 0.7f);
    }

    public void Hop(Vector2 hopTargetPos, float height = HOP_HEIGHT_BASE, bool rotative=false)
    {
        _hopStartPos = transform.position;
        _hopTargetPos = hopTargetPos;
        //Debug.Log("Hop " + _hopStartPos + ", " + _hopTargetPos);
        _hopTime = 0;
        _hopDuration = HOP_DURATION_BASE * (height / HOP_HEIGHT_BASE);
        _hopHeight = height;
        _isHopping = true;
        _isRotativeHop = rotative;

        SetFlipped(_hopTargetPos.x < _hopStartPos.x);
    }

    private void SetFlipped(bool flipped)
    {
        if (flipped)
        {
            transform.localRotation = Quaternion.Euler(0f, 180f, 0f);
        }
        else
        {
            transform.localRotation = Quaternion.identity;
        }
    }

    void RandomHop()
    {
        Vector2 pos = transform.position;
        Vector2 ss = GetComponent<SpriteRenderer>().bounds.size;
        Vector2 hopPos = new Vector2(pos.x, pos.y) + new Vector2(Random.Range(-ss.x, ss.x), Random.Range(-ss.y, ss.y));
        Hop(hopPos);
    }

    private static float HopCurve(float t)
    {
        return 1f - Mathf2.Sq(2*t - 1);
    }

    public void Kill(bool autoHop = true)
    {
        GetComponent<SpriteRenderer>().sprite = deadSprite;
        _alive = false;
        SpreadPanic(transform.position, PANIC_RADIUS); // Before changing layer
        gameObject.layer = LayerMask.NameToLayer("DeadBunnies");
        if(autoHop)
            RandomHop();
        --aliveCount;
        if (speedParticles != null)
            speedParticles.enableEmission = false;
        GameManager.instance.OnBunnyDied();
        if(GetComponent<BunnyController>() != null)
        {
            GameManager.instance.GameOver();
        }
        else
        {
            var cmp = gameObject.AddComponent<DestroyAfterTime>();
            cmp.time = 10f;
            cmp.fadeTime = 2f;
        }
    }

    void SpreadPanic(Vector2 pos, float radius)
    {
        var hits = Physics2D.OverlapCircleAll(pos, radius, 1 << gameObject.layer);
        //Debug.Log("Panic at " + pos + " radius " + radius + " layer " + gameObject.layer);
        for(int i = 0; i < hits.Length; ++i)
        {
            Collider2D c = hits[i];
            BunnyAI ai = c.GetComponent<BunnyAI>();
            if(ai != null)
            {
                ai.Panic();
            }
        }
    }

    public Vector2 ClampHopPosition(Vector2 pos)
    {
        var fieldBounds = Field.instance.bounds;
        fieldBounds.size *= 0.8f;
        pos.x = Mathf.Clamp(pos.x, fieldBounds.min.x, fieldBounds.max.x);
        pos.y = Mathf.Clamp(pos.y, fieldBounds.min.y, fieldBounds.max.y);
        return pos;
    }

    void OnDestroy()
    {
        --count;
        if(_alive)
        {
            --aliveCount;
        }
    }
}


