﻿using UnityEngine;
using System.Collections;

public class BunnyAI : MonoBehaviour
{
    private const float RANDOM_HOP_TIME = 1f;

    private Bunny _bunny;
    private Bunny bunny
    {
        get
        {
            if (_bunny == null)
                _bunny = GetComponent<Bunny>();
            return _bunny;
        }
    }

    private float _timeBeforeRandomHop;
    private float _panicTime;

    void Awake()
    {
        RandomizeTimeBeforeRandomHop();
    }

    void Start()
    {
    }

    public void Panic(float duration = 2f)
    {
        _panicTime = duration;
    }

    void RandomizeTimeBeforeRandomHop()
    {
        _timeBeforeRandomHop = RANDOM_HOP_TIME * Random.Range(1f, 2f);
    }

    void Update()
    {
        if (bunny.alive && !bunny.isHopping)
        {
            // Attraction
            bool cancelRandomHops = false;
            var avatar = BunnyController.instance;
            if (avatar != null && avatar.attractiveTime > 0f)
            {
                Vector2 pos = transform.position;
                Vector2 targetPos = avatar.transform.position;

                float d = Vector2.Distance(targetPos, pos);
                if (d < 3f)
                {
                    cancelRandomHops = true;
                    Vector2 v = (targetPos - pos).normalized;
                    pos = bunny.ClampHopPosition(pos + v * GetHopSize());
                    bunny.Hop(pos);
                }
            }

            if(!cancelRandomHops)
            {
                // Random hops
                _timeBeforeRandomHop -= Time.deltaTime;
                _panicTime -= Time.deltaTime;
                if (_timeBeforeRandomHop < 0 || _panicTime > 0)
                {
                    RandomizeTimeBeforeRandomHop();
                    RandomHop();
                }
            }
        }
    }

    float GetHopSize()
    {
        return GetComponent<SpriteRenderer>().bounds.size.x;
    }

    void RandomHop()
    {
        Vector2 pos = transform.position;
        Vector2 ss = GetComponent<SpriteRenderer>().bounds.size;

        if (_panicTime > 0)
            ss *= 1.5f;

        Vector2 hopPos = pos;
        var fieldBounds = Field.instance.bounds;

        for(int i = 0; i < 10; ++i)
        {
            hopPos = new Vector2(pos.x, pos.y) + new Vector2(Random.Range(-ss.x, ss.x), Random.Range(-ss.y, ss.y));
            if (fieldBounds.Contains(hopPos))
                break;
        }

        hopPos = bunny.ClampHopPosition(hopPos);

        bunny.Hop(hopPos);
    }

}

