﻿using UnityEngine;
using System.Collections;

public class Field : MonoBehaviour
{
    public static Field instance;

    public GameObject carrotPrefab;
    public GameObject speedCarrotPrefab;
    public GameObject loveCarrotPrefab;

    public int maxCarrots;
    public float carrotSpawnInterval = 1f;

    [HideInInspector] public int carrotCount;

    private float _timeBeforeSpawn = 0f;

    private SpriteRenderer _spriteRenderer;

    public Bounds bounds
    {
        get
        {
            Bounds b = spriteRenderer.bounds;
            b.size = b.size + new Vector3(0, 0, 10f); // We are in 2D!
            return b;
        }
    }

    private SpriteRenderer spriteRenderer
    {
        get
        {
            if (_spriteRenderer == null)
                _spriteRenderer = GetComponent<SpriteRenderer>();
            return _spriteRenderer;
        }
    }

    void Awake()
    {
        instance = this;
    }
	
	void Update ()
    {
        int currentMaxCarrots = Mathf.Clamp(Bunny.aliveCount + 1, 0, maxCarrots);
        if (carrotCount < currentMaxCarrots)
        {
            _timeBeforeSpawn -= Time.deltaTime;
            if(_timeBeforeSpawn <= 0)
            {
                SpawnCarrot();
                RandomizeTimeBeforeSpawn();
            }
        }
	}

    void SpawnCarrot()
    {
        if(Bunny.aliveCount < 5)
        {
            Spawn(carrotPrefab);
        }
        else
        {
            float r = Random.Range(0f, 1f);
            if (r < 0.85f)
                Spawn(carrotPrefab);
            else if (r < 0.95f)
                Spawn(speedCarrotPrefab);
            else
                Spawn(loveCarrotPrefab);
        }
    }

    void RandomizeTimeBeforeSpawn()
    {
        _timeBeforeSpawn = carrotSpawnInterval * Random.Range(0.7f, 1.3f);
    }

    void Spawn(GameObject prefab)
    {
        Vector2 pos = GetRandomPosition();
        Instantiate(prefab, pos, Quaternion.identity);
    }

    public Vector2 GetRandomPosition(float normalizedPadding = 0.2f)
    {
        SpriteRenderer sprite = GetComponent<SpriteRenderer>();
        Bounds b = sprite.bounds;

        // Pad a little (OMAGAD A COMMENTTTTT)
        b.size = b.size * (1f - normalizedPadding);

        Vector2 pos = new Vector2(
            Random.Range(b.min.x, b.max.x),
            Random.Range(b.min.y, b.max.y)
        );

        return pos;
    }
}
