﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class GameManager : MonoBehaviour
{
    public static GameManager instance;

    public GameObject gameOverElement;
    public Text statsLabel;
    public Text titleLabel;

    public BunnyCountTriggerInfo[] bunnyCountTriggers;

    private float _startTime;

    //private int _previousAliveCount;
    private int _startBornCount;

	void Awake()
    {
        instance = this;
	}

    void Start()
    {
        _startBornCount = Bunny.bornCount;

        gameOverElement.gameObject.SetActive(false);
        titleLabel.gameObject.SetActive(true);
    }

    public void HideTitle()
    {
        titleLabel.gameObject.SetActive(false);
    }

    public void GameOver()
    {
        gameOverElement.gameObject.SetActive(true);
        statsLabel.gameObject.SetActive(true);

        int seconds = (int)(Time.realtimeSinceStartup - _startTime);

        int minutes = seconds / 60;
        seconds = seconds % 60;

        int bornCount = Bunny.bornCount - _startBornCount;

        statsLabel.text = "You gave birth to " + bornCount + " cuties in " + minutes + "min " + seconds + "s\n"
            + "Top number on screen: " + Globals.instance.peakAliveBunnyCount;
    }
	
	void Update ()
    {
        if(titleLabel.gameObject.activeSelf && Input.GetMouseButtonDown(0))
        {
            HideTitle();
            _startTime = Time.realtimeSinceStartup;
        }
        if(gameOverElement.activeSelf)
        {
            if(Input.GetMouseButtonDown(0) || Input.GetKeyDown(KeyCode.Return))
            {
                NewGame();
            }
        }
        // Quit in fullscreen
        if(Input.GetKeyDown(KeyCode.Escape))
        {
            Application.Quit();
        }
        //if(Bunny.aliveCount == 0 && _previousAliveCount != 0)
        //{
        //}
        //_previousAliveCount = Bunny.aliveCount;
	}

    void NewGame()
    {
        _startTime = Time.realtimeSinceStartup;
        Application.LoadLevel(Application.loadedLevel);
    }

    public void OnBunnyBorn()
    {
        foreach(BunnyCountTriggerInfo t in bunnyCountTriggers)
        {
            if(Bunny.aliveCount == t.bunnyCount)
            {
                if(!t.enemy.activeSelf)
                    t.enemy.SetActive(true);
            }
        }
    }

    public void OnBunnyDied()
    {

    }
}

[System.Serializable]
public class BunnyCountTriggerInfo
{
    public GameObject enemy;
    public int bunnyCount;
}

