﻿using UnityEngine;
using System.Collections;

public class Truck : MonoBehaviour
{
    private const float ANIM_DURATION = 5f;

    public AnimationCurve dashCurve;
    public int bunnyCount = 30;
    public float callTimeIntervalMin = 10f;
    public float callTimeIntervalMax = 20f;

    public AudioClip motorSound;
    public AudioClip hitSound;

    private float _time;
    private bool _running = false;
    private float _timeBeforeNextCall;
    private Vector3 _startPos;
    private Vector3 _targetPos;

	void Start ()
    {
        //Call();
	}
	
    public void Call()
    {
        _time = 0;
        _running = true;

        Bounds fb = Field.instance.bounds;
        float y = 0.7f * Random.Range(fb.min.y, fb.max.y);
        float x = Random.Range(0f, 1f) < 0.5f ? -6.5f : 6.5f;

        float fieldWidth = Field.instance.bounds.size.x;

        _startPos = new Vector3(x, y, transform.position.z);
        _targetPos = _startPos + (1.6f * fieldWidth * (x > 0 ? -1f : 1f)) * Vector3.right;

        transform.position = _startPos;
        SetFlipped(x > 0);

        GetComponent<AudioSource>().PlayOneShot(motorSound);
    }

    private void SetFlipped(bool flipped)
    {
        if (flipped)
        {
            transform.localRotation = Quaternion.Euler(0f, 180f, 0f);
        }
        else
        {
            transform.localRotation = Quaternion.identity;
        }
    }

    void Update()
    {
        if(_running)
        {
            _time += Time.deltaTime;
            float r = _time / ANIM_DURATION;

            float tx = dashCurve.Evaluate(r);

            Vector3 pos = Vector3.Lerp(_startPos, _targetPos, tx);
            //pos.x = 1.5f * fieldWidth * (tx - 0.5f);
            transform.position = pos;

            if (r > 1f)
            {
                _timeBeforeNextCall = Random.Range(callTimeIntervalMin, callTimeIntervalMax);
                _running = false;
            }
        }
        else
        {
            if(Bunny.aliveCount > bunnyCount)
            {
                _timeBeforeNextCall -= Time.deltaTime;
                if (_timeBeforeNextCall <= 0)
                {
                    Call();
                }
            }
        }
    }

    void OnTriggerEnter2D(Collider2D c)
    {
        if(_time / ANIM_DURATION > 0.25f)
        {
            Bunny b = c.GetComponent<Bunny>();
            if(b != null && b.alive)
            {
                b.Kill(false);
                Vector3 dir = transform.right;
                b.Hop(b.transform.position + dir * 8f, 3f, true); // BENG
                GetComponent<AudioSource>().PlayOneShot(hitSound);
            }
        }
    }
}


